// Copyright 2020 Andy Grove
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

//! Ballista is a distributed compute platform implemented in Rust and based on Apache Arrow, with
//! strong support for multiple programming languages and also provides interoperability with Apache
//! Spark.

#![feature(async_closure)]

pub use arrow;
pub use datafusion;
pub use flight;
pub use parquet;
pub use sqlparser;

// include the generated protobuf source as a submodule
#[allow(clippy::all)]
pub mod protobuf {
    include!(concat!(env!("OUT_DIR"), "/ballista.protobuf.rs"));
}

pub const BALLISTA_VERSION: &str = env!("CARGO_PKG_VERSION");

pub mod dataframe;
pub mod distributed;
pub mod error;
pub mod execution;
pub mod serde;

#[macro_use]
pub mod utils;
